#version 120

uniform sampler2D texture;
uniform float time;

varying vec2 texCoord;
varying vec4 vertexColor;

// Parameters for VHS effects
const float scanlineIntensity = 0.1;
const float noiseIntensity = 0.05;
const float chromaticAberrationIntensity = 0.005;
const float noiseSpeed = 5.0; // Controls the speed of noise movement

// Function to generate pseudo-random value
float rand(vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main() {
    vec2 uv = texCoord;
    
    // Chromatic aberration
    vec2 redOffset = vec2(chromaticAberrationIntensity, 0.0);
    vec2 greenOffset = vec2(0.0, 0.0);
    vec2 blueOffset = vec2(-chromaticAberrationIntensity, 0.0);
    
    vec4 colorDistortion;
    colorDistortion.r = texture2D(texture, uv + redOffset).r;
    colorDistortion.g = texture2D(texture, uv + greenOffset).g;
    colorDistortion.b = texture2D(texture, uv + blueOffset).b;
    colorDistortion.a = texture2D(texture, uv).a;
    
    // Scan lines
    float scanline = sin(uv.y * 800.0) * 0.5 + 0.5;
    colorDistortion.rgb *= mix(1.0, scanline, scanlineIntensity);
    
    // Moving static/noise overlay
    vec2 noiseUV = uv + vec2(time * 0.1 * noiseSpeed, time * 0.2 * noiseSpeed);
    float noise = rand(noiseUV);

    colorDistortion.rgb = mix(colorDistortion.rgb, vec3(noise), noiseIntensity);
    
    gl_FragColor = colorDistortion * vertexColor;
}
